<?php
/**
 * CSV インポート・エクスポート補完クラス
 *
 * @author Tsue Shogo
 * @version	1.1.0
 *	1.0.0	2014/03/27 15:30	新規作成
 *	1.1.0	2015/11/09 06:13	バグ修正
 */
class Csv {

/**
 * セパレータ
 */
	public $sep = ',';

/**
 * 改行コード
 */
	public $br = "\n";

/**
 * CSV1値からCSV文字列を出力する
 *
 * @param string 	$value	CSV1値
 * @return string 	CSV要素文字列
 */
	public function outCsvValue($value) {
		// 初期準備
		$str = $value;

		// 値にダブルクォーテーションがある場合は2重ダブルクォーテーションに置換する
		$str = str_replace('"', '""', $str);

		// 値全体をダブルクォーテーションで囲む
		$str = '"'.$str.'"';

		return $str;
	}

/**
 * CSV1行からCSV1行文字列を出力する
 *
 * @param array 	$values 	CSV1行
 * @return string 	CSV1行文字列
 */
	public function outCsvRecord($values) {
		// 初期準備
		$strs = array();

		// 値を半角カンマで連結する
		if ($values && is_array($values)) {
			foreach ($values as $value) {
				$strs[] = $this->outCsvValue($value);
			}
		}
		$str = implode($this->sep, $strs);

		return $str;
	}

/**
 * CSV複数行からCSV複数行文字列を出力する
 *
 * @param array 	$records 	CSV複数行
 * @return string 	CSV複数行
 */
	public function outCsvRecords($records) {
		// 初期準備
		$str = '';

		// 複数行作成
		if ($records && is_array($records)) {
			foreach ($records as $record) {
				$str .= $this->outCsvRecord($record) . $this->br;
			}
		}

		return $str;
	}

/**
 * CSV値を解釈する
 *
 * @param string 	$value_str 	CSV値
 * @return string 	CSV1値
 */
	public function parseCsvValueStr($value_str) {
		// 初期準備
		$value = $value_str;

		if (
			// 両端がダブルクォーテーションの場合は取り除く
			$value && substr($value, 0, 1) === '"' && substr($value, -1, 1) === '"'
			// ただし、両端が2重ダブルクォーテーションにしてる場合は無視する
			&& (strlen($value) < 4 || substr($value, 0, 2) !== '""' || substr($value, -1, 2) !== '""')
		) {
			$value = substr($value, 1, strlen($value) - 2);
		}
		// 2重ダブルクォーテーションは1重に置換する
		$value = str_replace('""', '"', $value);

		return $value;
	}

/**
 * CSV1レコードを文字列から解釈する
 *
 * @param string 	$str 	CSV1レコード
 * @return array 	CSV1レコード
 */
	public function parseCsvRecordStr($record_str) {
		// 初期準備
		$values = array();
		$tmp_piece_str = '';

		// レコード文字列を行文字列に分割
		$row_strs = explode($this->sep, $record_str);
		foreach ($row_strs as $i => $row_str) {

			// 1行をピースに分割
			$piece_strs = explode($this->sep, $row_str);
			foreach ($piece_strs as $j => $piece_str) {

				// ピースを退避
				$tmp_piece_str = $tmp_piece_str . ($tmp_piece_str ? ($j ? $this->sep : ($i ? $this->br : '')) : '') . $piece_str;
				$piece_str = $tmp_piece_str;

				// ピースに含まれるダブルクォーテーションの個数を調べる
				if (substr_count($piece_str, '"') % 2 === 0) {
					$value_str = $piece_str;
					$value = $this->parseCsvValueStr($value_str);
					if ($value === false) {
						return false;
					}
					$values[] = $value;
					$tmp_piece_str = '';
				}
			}

		}
		if ($tmp_piece_str) {
			return false;
		}

		return $values;
	}

/**
 * CSV複数レコード文字列を解釈する
 *
 * @param string 	$str 	文字列
 * @return array 	CSV複数行
 */
	public function parseCsvRecordsStr($records_str) {
		// 初期準備
		$records = array();
		$tmp_row_strs = array();

		// 行ループ
		$row_strs = explode($this->br, $records_str);
		foreach ($row_strs as $i => $row_str) {

			// 退避1行の追加
			$tmp_row_strs[] = $row_str;
			$row_str = implode($this->br, $tmp_row_strs);

			// 1row≠1record
			if (substr_count($row_str, '"') % 2) {
				continue;
			}

			// 1row=1record
			$record_str = $row_str;
			$tmp_row_strs = array();
			$record = $this->parseCsvRecordStr($record_str);
			if ($record === false) {
				return false;
			}
			$records[] = $record;

		}
		if ($tmp_row_strs) {
			return false;
		}

		return $records;
	}

/**
 * CSV1行をファイルから読み込む
 *
 * @param int 	$fp 	CSVファイル
 * @return array 	CSV1行
 */
	public function parseCsvFile($fp) {
		// 初期準備
		$tmp_row_str = '';
		$record = null;

		// ループ
		while ($row_str = fgets($fp)) {

			// 行取得
			$tmp_row_str .= ($tmp_row_str ? $this->br : '').substr($row_str, 0, -1);
			$row_str = $tmp_row_str;

			if (substr_count($row_str, '"') % 2) {
				continue;
			} else {
				$record_str = $row_str;
				$record = $this->parseCsvRecordStr($record_str);
				$tmp_row_str = '';
				break;
			}
		}
		if ($tmp_row_str) {
			return false;
		}

		return $record;
	}
}